@@ -1,7 +1,9 @@
 import click
-from cosmoglobe.plot.click_plotting import commands_plotting
-from cosmoglobe.h5.click_h5 import commands_h5
+
 from cosmoglobe.fits.click_fits import commands_fits
+from cosmoglobe.release.commands_hdf import commands_hdf
+from cosmoglobe.h5.click_h5 import commands_h5
+from cosmoglobe.plot.click_plotting import commands_plotting
 
 CONTEXT_SETTINGS = dict(max_content_width=100)
 
@@ -9,8 +11,8 @@ CONTEXT_SETTINGS = dict(max_content_width=100)
 def cli():
     pass
 
-cli = click.CommandCollection(sources=[commands_plotting, commands_h5, commands_fits], context_settings=CONTEXT_SETTINGS)
+cli = click.CommandCollection(sources=[commands_plotting, commands_h5, commands_fits, commands_hdf], context_settings=CONTEXT_SETTINGS)
 
 if __name__ == '__main__':
     cli()
-    
\ No newline at end of file
+    
