@@ -1,15 +1,18 @@
 from __future__ import annotations
-from pathlib import Path
+
 import textwrap
-from typing import Any, Dict, Generator, List, Optional, Sequence, Union
+from pathlib import Path
+from typing import Any, Generator, Optional, Sequence
 
 import h5py
+import healpy as hp
 import numpy as np
 
-from cosmoglobe.h5 import ChainVersion, PARAMETER_GROUP_NAME
+from cosmoglobe.h5 import PARAMETER_GROUP_NAME, ChainVersion
+from cosmoglobe.h5._alms import unpack_alms as unpack_alms_
 from cosmoglobe.h5._alms import unpack_alms_from_chain
-from cosmoglobe.h5._decorators import validate_key, validate_samples, unpack_alms
-from cosmoglobe.h5._exceptions import ChainFormatError, ChainSampleError, ChainKeyError
+from cosmoglobe.h5._decorators import unpack_alms, validate_key, validate_samples
+from cosmoglobe.h5._exceptions import ChainFormatError, ChainKeyError, ChainSampleError
 from cosmoglobe.sky.components._labels import SkyComponentLabel
 
 
@@ -20,7 +23,7 @@ class Chain:
     Cosmoglobe chain files.
     """
 
-    def __init__(self, path: Union[str, Path], burn_in: Optional[int] = None) -> None:
+    def __init__(self, path: str | Path, burn_in: Optional[int] = None) -> None:
         """Validate and initialize the Chain object.
 
         Parameters
@@ -57,7 +60,7 @@ class Chain:
                 if group in [label.value for label in SkyComponentLabel]
             ]
 
-            parameters: Dict[str, Dict[str, Any]] = {}
+            parameters: dict[str, dict[str, Any]] = {}
             if version is ChainVersion.NEW:
                 for component, group in file[PARAMETER_GROUP_NAME].items():
                     parameters[component] = {}
@@ -78,7 +81,7 @@ class Chain:
         self._version = version
 
     @property
-    def samples(self) -> List[str]:
+    def samples(self) -> list[str]:
         """List of all samples in the chain."""
 
         return self._samples
@@ -90,13 +93,13 @@ class Chain:
         return len(self.samples)
 
     @property
-    def components(self) -> List[str]:
+    def components(self) -> list[str]:
         """List of the sky components in the chain."""
 
         return self._components
 
     @property
-    def parameters(self) -> Dict[str, Dict[str, Any]]:
+    def parameters(self) -> dict[str, dict[str, Any]]:
         """Dictionary of the parameters in the parameter group in the chain."""
 
         return self._parameters
@@ -135,7 +138,8 @@ class Chain:
         self,
         key: str,
         *,
-        samples: Optional[Union[range, int, Sequence[int]]] = None,
+        samples: Optional[range | int | Sequence[int]] = None,
+        unpack: bool = True,
     ) -> Any:
         """Returns the value of an key for all samples.
 
@@ -147,6 +151,9 @@ class Chain:
         samples
             An int or a range of samples for which to return the value. If
             None, all samples in the chain are used.
+        unpack
+            If True, alms are unpacked from Commander format to healpy format.
+            Default is True
 
         Returns
         -------
@@ -166,10 +173,47 @@ class Chain:
         self,
         key: str,
         *,
-        samples: Optional[Union[range, int, Sequence[int]]] = None,
+        samples: Optional[range | int | Sequence[int]] = None,
+        unpack: bool = True,
     ) -> Any:
         """Returns the mean of an key over all samples.
 
+        Parameters
+        ----------
+        key
+            The path to an item that has been sampled in the chain, e.g
+            'dust/amp_alm'.
+        samples
+            An int or a range of samples to average over. If None, all
+            samples in the chain are used.
+        unpack
+            If True, alms are unpacked from Commander format to healpy format.
+            Default is True
+            
+        Returns
+        -------
+        value
+            The averaged value of the key over all samples.
+        """
+        with h5py.File(self.path, "r") as file:
+            value = file[f"{samples[0]}/{key}"][()]
+            dtype = value.dtype.type
+            if len(samples) > 1:
+                for sample in samples[1:]:
+                    value += file[f"{sample}/{key}"][()]
+        return dtype(value / len(samples))  # Converting back to original dtype
+
+    @validate_key
+    @validate_samples
+    def stddev(
+        self,
+        key: str,
+        *,
+        alm2map=False,
+        samples: Optional[range | int | Sequence[int]] = None,
+    ) -> Any:
+        """Returns the stddev of an key over all samples.
+
         Parameters
         ----------
         key
@@ -191,8 +235,30 @@ class Chain:
             if len(samples) > 1:
                 for sample in samples[1:]:
                     value += file[f"{sample}/{key}"][()]
+        mu = dtype(value / len(samples))  # Converting back to original dtype
 
-        return dtype(value / len(samples))  # Converting back to original dtype
+        # Calculate in map-space if alm2map
+        if alm2map:
+            comp, quantity = key.split("/")
+            nside = self.parameters[comp]["nside"]
+            pol = True if quantity.startswith("amp") else False
+            fwhm = self.parameters[comp]["fwhm"]
+            lmax = 3*nside
+            mu = hp.alm2map(unpack_alms_(mu, lmax), nside=nside, lmax=lmax, fwhm=fwhm, pol=pol, pixwin=True,)
+
+
+        with h5py.File(self.path, "r") as file: 
+            x = file[f"{samples[0]}/{key}"][()] 
+            if alm2map: x = hp.alm2map(unpack_alms_(x, lmax), nside=nside, lmax=lmax, fwhm=fwhm, pol=pol, pixwin=True,)
+            dtype = x.dtype.type
+            numerator = (x - mu)**2
+            if len(samples) > 1:
+                for sample in samples[1:]:
+                    x = file[f"{sample}/{key}"][()]
+                    if alm2map: x = hp.alm2map(unpack_alms_(x, lmax), nside=nside, lmax=lmax, fwhm=fwhm, pol=pol, pixwin=True,)
+                    numerator += (x - mu)**2
+
+        return dtype(np.sqrt(numerator/len(samples)))  # Converting back to original dtype
 
     @validate_key
     @validate_samples
@@ -200,7 +266,7 @@ class Chain:
         self,
         key: str,
         *,
-        samples: Optional[Union[range, int, Sequence[int]]] = None,
+        samples: Optional[range | int | Sequence[int]] = None,
     ) -> Generator:
         """Returns a generator to be used in a for loop.
 
@@ -231,13 +297,16 @@ class Chain:
 
     @validate_key
     @unpack_alms
-    def __getitem__(self, key: str) -> Any:
+    def __getitem__(self, key: str, *, unpack: bool = True) -> Any:
         """Returns the value of a key from the chain.
 
         Parameters
         ----------
         key
             The *full* path to an item in the chain.
+        unpack
+            If True, alms are unpacked from Commander format to healpy format.
+            Default is True
 
         Returns
         -------
@@ -254,7 +323,7 @@ class Chain:
                     return item.asstr()[()]
                 return item[()]
 
-    def _format_samples(self, samples: Union[List[int], int]) -> Union[List[str], str]:
+    def _format_samples(self, samples: list[int] | int) -> list[str] | str:
         """Converts a range to the string format of the samples in the chain."""
 
         leading_zeros = len(self.samples[0])
@@ -308,7 +377,7 @@ class Chain:
     @validate_samples
     def copy(
         self,
-        samples: Union[int, Sequence[int], range] = -1,
+        samples: int | Sequence[int] | range = -1,
         new_name: Optional[str] = None,
     ) -> None:
         """Creates a copy of the chain with a single or multiple samples."""
@@ -316,27 +385,23 @@ class Chain:
         if new_name is None:
             new_name = self.path.stem + "_copy.h5"
 
-        if Path(new_name).is_file():
-            print(f"You are about to overwrite {new_name}. Would you like to continue? [y/n]")
-            if not input() in ["y", "Y"]:
-                exit()
-
-        with h5py.File(new_name, "w") as new_chain:
+        with h5py.File(new_name, "x") as new_chain:
             with h5py.File(self.path, "r") as chain:
                 for idx, sample in enumerate(samples):
                     group = chain[sample]
-                    chain.copy(source=group, dest=new_chain, name=self._format_samples(idx))
+                    chain.copy(
+                        source=group, dest=new_chain, name=self._format_samples(idx)
+                    )
 
                 parameter_group = chain["parameters"]
                 chain.copy(
                     source=parameter_group, dest=new_chain, name=parameter_group.name
                 )
 
-
     def combine(
         self,
         other_chain: Chain,
-        group_list: List[str],
+        group_list: Sequence[str],
         new_name: Optional[str] = None,
     ) -> None:
         """Creates a new chainfile that combines specific groups from two chains."""
@@ -360,20 +425,36 @@ class Chain:
                     if group in new_chain[sample].keys():
                         del new_chain[f"{sample}/{group}"]
                     group_to_copy = chain[f"{sample}/{group}"]
-                    chain.copy(source=group_to_copy, dest=new_chain, name=group_to_copy.name)
+                    chain.copy(
+                        source=group_to_copy, dest=new_chain, name=group_to_copy.name
+                    )
 
                     if group in chain[f"parameters"].keys():
                         del new_chain[f"parameters/{group}"]
                         param_to_copy = chain[f"parameters/{group}"]
-                        chain.copy(source=param_to_copy, dest=new_chain, name=param_to_copy.name)
+                        chain.copy(
+                            source=param_to_copy,
+                            dest=new_chain,
+                            name=param_to_copy.name,
+                        )
 
 
 def copy_chain(
-    chain: Union[str, Path, Chain],
-    samples: Union[int, Sequence[int], range] = -1,
+    chain: str | Path | Chain,
+    samples: int | Sequence[int] | range = -1,
     new_name: Optional[str] = None,
 ) -> None:
-    """Creates a copy of the chain with a single or multiple samples."""
+    """Creates a copy of the chain with a single or multiple samples.
+
+    Parameters
+    ----------
+    chain
+        Path to the chain file or the `Chain` object to copy.
+    samples
+        Samples to copy. Can be an int, a list of ints or a python range object.
+    new_name
+        Name of the chain copy. If None, a default is "{chain.name}_copy.h5"
+    """
 
     if not isinstance(chain, Chain):
         chain = Chain(chain)
@@ -381,17 +462,35 @@ def copy_chain(
     chain.copy(samples=samples, new_name=new_name)
 
 
-def combine_chain(
-    chain: Union[str, Path, Chain],
-    other_chain: Union[str, Path, Chain],
-    group_list: List[str],
+def combine_chains(
+    chain: str | Path | Chain,
+    other_chain: str | Path | Chain,
+    group_list: Sequence[str],
     new_name: Optional[str] = None,
 ) -> None:
-    """Creates a new chainfile that combines specific groups from two chains."""
+    """Creates a new chainfile that combines specific groups from two chains.
+
+    The new file will contain all content from `chain`, except for the content
+    within the groups in the `group_list`, which are taken from `other_chain`
+    instead.
+
+    Parameters
+    ----------
+    chain
+        Path to chain file. This chain defines all the chain whos content you
+        want to overwrite in a new combined file.
+    other_chain
+        Path to chain file.  This chain contains the groups you want to
+        overwrite in `chain`.
+    group_list
+        List of hdf5 groups that will be overwritten in the new chainfile.
+    new_name
+        Name of the chain copy. If None, a default is "{chain.name}_copy.h5"
+    """
 
     if not isinstance(chain, Chain):
         chain = Chain(chain)
     if not isinstance(other_chain, Chain):
         other_chain = Chain(other_chain)
 
-    chain.combine(other_chain, group_list=group_list)
\ No newline at end of file
+    chain.combine(other_chain, group_list=group_list, new_name=new_name)
