@@ -3,3 +3,5 @@ from .gnomonic import gnom
 from .histogram import hist
 from .traceplot import trace
 from .spectrum import spec
+from .plottools import standalone_colorbar
+from .temp_newvisufunc import projview
